@@ -57,10 +57,13 @@ main (int    argc,
 
   instance = view->create (view);
   context = view->get_context (instance);
-  do_output (context, NULL, NULL, NULL, NULL, &width, &height);
+  width = height = 1;
+  surface = view->create_surface (instance, width, height);
+  view->render (instance, surface, context, &width, &height, NULL);
+  view->destroy_surface (instance, surface);
   surface = view->create_surface (instance, width, height);
   for (run = 0; run < MAX(1,opt_runs); run++)
-    view->render (instance, surface, context, width, height, NULL);
+    view->render (instance, surface, context, &width, &height, NULL);
 
   if (opt_output)
     {
@@ -86,7 +89,7 @@ main (int    argc,
 
 	      convert_argv[2] = opt_output;
 
-	      if (!g_spawn_async_with_pipes (NULL, (gchar **)convert_argv, NULL,
+	      if (!g_spawn_async_with_pipes (NULL, (gchar **)(void*)convert_argv, NULL,
 					     G_SPAWN_DO_NOT_REAP_CHILD |
 					     G_SPAWN_SEARCH_PATH |
 					     G_SPAWN_STDOUT_TO_DEV_NULL |
@@ -123,7 +126,7 @@ main (int    argc,
 	      if (!state)
 		break;
 
-	      view->render (instance, surface, context, width, height, state);
+	      view->render (instance, surface, context, &width, &height, state);
 	    }
 
 	  if (view->destroy_window)
@@ -141,7 +144,7 @@ main (int    argc,
 	    fail ("%s viewer backend does not support displaying or writing", view->name);
 	  display_argv[2] = title;
 
-	  if (!g_spawn_async_with_pipes (NULL, (gchar **)display_argv, NULL,
+	  if (!g_spawn_async_with_pipes (NULL, (gchar **)(void*)display_argv, NULL,
 					 G_SPAWN_DO_NOT_REAP_CHILD |
 					 G_SPAWN_SEARCH_PATH |
 					 G_SPAWN_STDOUT_TO_DEV_NULL |
