@@ -625,9 +625,16 @@ compute_distance (const PangoFontDescription *a,
  * @new_match: a #PangoFontDescription
  *
  * Determines if the style attributes of @new_match are a closer match
- * for @desc than @old_match, or if @old_match is %NULL, determines if
- * @new_match is a match at all. Approximate matching is done for
- * weight and style; other attributes must match exactly.
+ * for @desc than those of @old_match are, or if @old_match is %NULL,
+ * determines if @new_match is a match at all.
+ * Approximate matching is done for
+ * weight and style; other style attributes must match exactly.
+ * Style attributes are all attributes other than family and size-related
+ * attributes.  Approximate matching for style considers PANGO_STYLE_OBLIQUE
+ * and PANGO_STYLE_ITALIC as matches, but not as good a match as when the
+ * styles are equal.
+ *
+ * Note that @old_match must match @desc.
  *
  * Return value: %TRUE if @new_match is a better match
  **/
@@ -817,7 +824,7 @@ pango_font_description_free  (PangoFontDescription  *desc)
  * @descs: a pointer to an array of #PangoFontDescription, may be %NULL
  * @n_descs: number of font descriptions in @descs
  *
- * Frees a list of font descriptions from pango_font_map_list_fonts()
+ * Frees an array of font descriptions.
  **/
 void
 pango_font_descriptions_free (PangoFontDescription **descs,
@@ -841,6 +848,7 @@ typedef struct
 
 static const FieldMap style_map[] = {
   { PANGO_STYLE_NORMAL, "" },
+  { PANGO_STYLE_NORMAL, "Roman" },
   { PANGO_STYLE_OBLIQUE, "Oblique" },
   { PANGO_STYLE_ITALIC, "Italic" }
 };
@@ -851,14 +859,25 @@ static const FieldMap variant_map[] = {
 };
 
 static const FieldMap weight_map[] = {
+  { PANGO_WEIGHT_THIN, "Thin" },
   { PANGO_WEIGHT_ULTRALIGHT, "Ultra-Light" },
+  { PANGO_WEIGHT_ULTRALIGHT, "Extra-Light" },
   { PANGO_WEIGHT_LIGHT, "Light" },
+  { PANGO_WEIGHT_BOOK, "Book" },
   { PANGO_WEIGHT_NORMAL, "" },
-  { 500, "Medium" },
+  { PANGO_WEIGHT_NORMAL, "Regular" },
+  { PANGO_WEIGHT_MEDIUM, "Medium" },
   { PANGO_WEIGHT_SEMIBOLD, "Semi-Bold" },
+  { PANGO_WEIGHT_SEMIBOLD, "Demi-Bold" },
   { PANGO_WEIGHT_BOLD, "Bold" },
   { PANGO_WEIGHT_ULTRABOLD, "Ultra-Bold" },
-  { PANGO_WEIGHT_HEAVY, "Heavy" }
+  { PANGO_WEIGHT_ULTRABOLD, "Extra-Bold" },
+  { PANGO_WEIGHT_HEAVY, "Heavy" },
+  { PANGO_WEIGHT_HEAVY, "Black" },
+  { PANGO_WEIGHT_ULTRAHEAVY, "Ultra-Heavy" },
+  { PANGO_WEIGHT_ULTRAHEAVY, "Extra-Heavy" },
+  { PANGO_WEIGHT_ULTRAHEAVY, "Ultra-Black" },
+  { PANGO_WEIGHT_ULTRAHEAVY, "Extra-Black" }
 };
 
 static const FieldMap stretch_map[] = {
@@ -913,9 +932,46 @@ field_matches (const gchar *s1,
 }
 
 static gboolean
-find_field (const FieldMap *map, int n_elements, const char *str, int len, int *val)
+parse_int (const char *word,
+	   size_t      wordlen,
+	   int        *out)
+{
+  char *end;
+  long val = strtol (word, &end, 10);
+  int i = val;
+
+  if (end != word && (end == word + wordlen) && val >= 0 && val == i)
+    {
+      if (out)
+        *out = i;
+
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+static gboolean
+find_field (const char *what,
+	    const FieldMap *map,
+	    int n_elements,
+	    const char *str,
+	    int len,
+	    int *val)
 {
   int i;
+  gboolean had_prefix = FALSE;
+
+  if (what)
+    {
+      i = strlen (what);
+      if (len > i && 0 == strncmp (what, str, i) && str[i] == '=')
+	{
+	  str += i + 1;
+	  len -= i + 1;
+	  had_prefix = TRUE;
+	}
+    }
 
   for (i=0; i<n_elements; i++)
     {
@@ -927,37 +983,38 @@ find_field (const FieldMap *map, int n_elements, const char *str, int len, int *
 	}
     }
 
+  if (!what || had_prefix)
+    return parse_int (str, len, val);
+
   return FALSE;
 }
 
 static gboolean
 find_field_any (const char *str, int len, PangoFontDescription *desc)
 {
-  gboolean found = FALSE;
-
-  if (g_ascii_strcasecmp (str, "Normal") == 0)
+  if (field_matches ("Normal", str, len))
     return TRUE;
 
 #define FIELD(NAME, MASK) \
   G_STMT_START { \
-  if (find_field (NAME##_map, G_N_ELEMENTS (NAME##_map), str, len, \
-		  desc ? (int *)&desc->NAME : NULL)) \
+  if (find_field (G_STRINGIFY (NAME), NAME##_map, G_N_ELEMENTS (NAME##_map), str, len, \
+		  desc ? (int *)(void *)&desc->NAME : NULL)) \
     { \
-      found = TRUE; \
       if (desc) \
 	desc->mask |= MASK; \
+      return TRUE; \
     } \
   } G_STMT_END
 
-  FIELD (style,   PANGO_FONT_MASK_STYLE);
-  FIELD (variant, PANGO_FONT_MASK_VARIANT);
   FIELD (weight,  PANGO_FONT_MASK_WEIGHT);
+  FIELD (style,   PANGO_FONT_MASK_STYLE);
   FIELD (stretch, PANGO_FONT_MASK_STRETCH);
+  FIELD (variant, PANGO_FONT_MASK_VARIANT);
   FIELD (gravity, PANGO_FONT_MASK_GRAVITY);
 
 #undef FIELD
 
-  return found;
+  return FALSE;
 }
 
 static const char *
@@ -969,7 +1026,7 @@ getword (const char *str, const char *last, size_t *wordlen)
     last--;
 
   result = last;
-  while (result > str && !g_ascii_isspace (*(result - 1)))
+  while (result > str && !g_ascii_isspace (*(result - 1)) && *(result - 1) != ',')
     result--;
 
   *wordlen = last - result;
@@ -1086,7 +1143,23 @@ pango_font_description_from_string (const char *str)
 
   if (str != last)
     {
+      int i;
+      char **families;
+
       desc->family_name = g_strndup (str, last - str);
+
+      /* Now sanitize it to trim space from around individual family names.
+       * bug #499624 */
+
+      families = g_strsplit (desc->family_name, ",", -1);
+
+      for (i = 0; families[i]; i++)
+	g_strstrip (families[i]);
+
+      g_free (desc->family_name);
+      desc->family_name = g_strjoinv (",", families);
+      g_strfreev (families);
+
       desc->mask |= PANGO_FONT_MASK_FAMILY;
     }
 
@@ -1094,26 +1167,26 @@ pango_font_description_from_string (const char *str)
 }
 
 static void
-append_field (GString *str, const FieldMap *map, int n_elements, int val)
+append_field (GString *str, const char *what, const FieldMap *map, int n_elements, int val)
 {
   int i;
   for (i=0; i<n_elements; i++)
     {
-      if (map[i].value == val)
+      if (map[i].value != val)
+        continue;
+
+      if (G_LIKELY (map[i].str && map[i].str[0]))
 	{
-	  if (map[i].str && map[i].str[0])
-	    {
-	      if (str->len > 0 && str->str[str->len -1] != ' ')
-		g_string_append_c (str, ' ');
-	      g_string_append (str, map[i].str);
-	    }
-	  return;
+	  if (G_LIKELY (str->len > 0 && str->str[str->len -1] != ' '))
+	    g_string_append_c (str, ' ');
+	  g_string_append (str, map[i].str);
 	}
+      return;
     }
 
-  if (str->len > 0 || str->str[str->len -1] != ' ')
+  if (G_LIKELY (str->len > 0 || str->str[str->len -1] != ' '))
     g_string_append_c (str, ' ');
-  g_string_append_printf (str, "%d", val);
+  g_string_append_printf (str, "%s=%d", what, val);
 }
 
 /**
@@ -1137,7 +1210,7 @@ pango_font_description_to_string (const PangoFontDescription  *desc)
 
   result = g_string_new (NULL);
 
-  if (desc->family_name && desc->mask & PANGO_FONT_MASK_FAMILY)
+  if (G_LIKELY (desc->family_name && desc->mask & PANGO_FONT_MASK_FAMILY))
     {
       const char *p;
       size_t wordlen;
@@ -1160,12 +1233,17 @@ pango_font_description_to_string (const PangoFontDescription  *desc)
 	g_string_append_c (result, ',');
     }
 
-  append_field (result, weight_map, G_N_ELEMENTS (weight_map), desc->weight);
-  append_field (result, style_map, G_N_ELEMENTS (style_map), desc->style);
-  append_field (result, stretch_map, G_N_ELEMENTS (stretch_map), desc->stretch);
-  append_field (result, variant_map, G_N_ELEMENTS (variant_map), desc->variant);
+#define FIELD(NAME, MASK) \
+  append_field (result, G_STRINGIFY (NAME), NAME##_map, G_N_ELEMENTS (NAME##_map), desc->NAME)
+
+  FIELD (weight,  PANGO_FONT_MASK_WEIGHT);
+  FIELD (style,   PANGO_FONT_MASK_STYLE);
+  FIELD (stretch, PANGO_FONT_MASK_STRETCH);
+  FIELD (variant, PANGO_FONT_MASK_VARIANT);
   if (desc->mask & PANGO_FONT_MASK_GRAVITY)
-    append_field (result, gravity_map, G_N_ELEMENTS (gravity_map), desc->gravity);
+    FIELD (gravity, PANGO_FONT_MASK_GRAVITY);
+
+#undef FIELD
 
   if (result->len == 0)
     g_string_append (result, "Normal");
@@ -1208,13 +1286,12 @@ pango_font_description_to_filename (const PangoFontDescription  *desc)
 
   result = pango_font_description_to_string (desc);
 
-  /* XXX This should be rewritten to read char-by-char instead
-   * of byte-by-byte, to be Unicode safe.
-   */
   p = result;
   while (*p)
     {
-      if (strchr ("-+_.", *p) == NULL && !g_ascii_isalnum (*p))
+      if (G_UNLIKELY ((guchar) *p >= 128))
+        /* skip over non-ASCII chars */;
+      else if (strchr ("-+_.", *p) == NULL && !g_ascii_isalnum (*p))
 	*p = '_';
       else
 	*p = g_ascii_tolower (*p);
@@ -1224,15 +1301,162 @@ pango_font_description_to_filename (const PangoFontDescription  *desc)
   return result;
 }
 
+
+static gboolean
+parse_field (const char *what,
+	     const FieldMap *map,
+	     int n_elements,
+	     const char *str,
+	     int *val,
+	     gboolean warn)
+{
+  gboolean found;
+  int len = strlen (str);
+
+  if (G_UNLIKELY (*str == '\0'))
+    return FALSE;
+
+  if (field_matches ("Normal", str, len))
+    {
+      /* find the map entry with empty string */
+      int i;
+
+      for (i = 0; i < n_elements; i++)
+        if (map[i].str[0] == '\0')
+	  {
+	    *val = map[i].value;
+	    return TRUE;
+	  }
+
+      *val = 0;
+      return TRUE;
+    }
+
+  found = find_field (NULL, map, n_elements, str, len, val);
+
+  if (!found && warn)
+    {
+	int i;
+	GString *s = g_string_new (NULL);
+
+	for (i = 0; i < n_elements; i++)
+	  {
+	    if (i)
+	      g_string_append_c (s, '/');
+	    g_string_append (s, map[i].str[0] == '\0' ? "Normal" : map[i].str);
+	  }
+
+	g_warning ("%s must be one of %s or a number",
+		   what,
+		   s->str);
+
+	g_string_free (s, TRUE);
+    }
+
+  return found;
+}
+
+#define FIELD(NAME, MASK) \
+  parse_field (G_STRINGIFY (NAME), NAME##_map, G_N_ELEMENTS (NAME##_map), str, (int *)(void *)NAME, warn)
+
+/**
+ * pango_parse_style:
+ * @str: a string to parse.
+ * @style: a #PangoStyle to store the result in.
+ * @warn: if %TRUE, issue a g_warning() on bad input.
+ *
+ * Parses a font style. The allowed values are "normal",
+ * "italic" and "oblique", case variations being
+ * ignored.
+ *
+ * Return value: %TRUE if @str was successfully parsed.
+ **/
+gboolean
+pango_parse_style (const char *str,
+		   PangoStyle *style,
+		   gboolean    warn)
+{
+  return FIELD (style,   PANGO_FONT_MASK_STYLE);
+}
+
+/**
+ * pango_parse_variant:
+ * @str: a string to parse.
+ * @variant: a #PangoVariant to store the result in.
+ * @warn: if %TRUE, issue a g_warning() on bad input.
+ *
+ * Parses a font variant. The allowed values are "normal"
+ * and "smallcaps" or "small_caps", case variations being
+ * ignored.
+ *
+ * Return value: %TRUE if @str was successfully parsed.
+ **/
+gboolean
+pango_parse_variant (const char   *str,
+		     PangoVariant *variant,
+		     gboolean	   warn)
+{
+  return FIELD (variant, PANGO_FONT_MASK_VARIANT);
+}
+
+/**
+ * pango_parse_weight:
+ * @str: a string to parse.
+ * @weight: a #PangoWeight to store the result in.
+ * @warn: if %TRUE, issue a g_warning() on bad input.
+ *
+ * Parses a font weight. The allowed values are "heavy",
+ * "ultrabold", "bold", "normal", "light", "ultraleight"
+ * and integers. Case variations are ignored.
+ *
+ * Return value: %TRUE if @str was successfully parsed.
+ **/
+gboolean
+pango_parse_weight (const char  *str,
+		    PangoWeight *weight,
+		    gboolean     warn)
+{
+  return FIELD (weight,  PANGO_FONT_MASK_WEIGHT);
+}
+
+/**
+ * pango_parse_stretch:
+ * @str: a string to parse.
+ * @stretch: a #PangoStretch to store the result in.
+ * @warn: if %TRUE, issue a g_warning() on bad input.
+ *
+ * Parses a font stretch. The allowed values are
+ * "ultra_condensed", "extra_condensed", "condensed",
+ * "semi_condensed", "normal", "semi_expanded", "expanded",
+ * "extra_expanded" and "ultra_expanded". Case variations are
+ * ignored and the '_' characters may be omitted.
+ *
+ * Return value: %TRUE if @str was successfully parsed.
+ **/
+gboolean
+pango_parse_stretch (const char   *str,
+		     PangoStretch *stretch,
+		     gboolean	   warn)
+{
+  return FIELD (stretch, PANGO_FONT_MASK_STRETCH);
+}
+
+
+
+
+/*
+ * PangoFont
+ */
+
 G_DEFINE_ABSTRACT_TYPE (PangoFont, pango_font, G_TYPE_OBJECT)
 
 static void
-pango_font_class_init (PangoFontClass *class)
+pango_font_class_init (PangoFontClass *class G_GNUC_UNUSED)
 {
 }
 
 static void
-pango_font_init (PangoFont *font)
+pango_font_init (PangoFont *font G_GNUC_UNUSED)
 {
 }
 
@@ -1414,10 +1638,18 @@ pango_font_get_metrics (PangoFont        *font,
 
 /**
  * pango_font_get_font_map:
- * @font: a #PangoFont
+ * @font: a #PangoFont, or %NULL
  *
  * Gets the font map for which the font was created.
  *
+ * Note that the font maintains a <firstterm>weak</firstterm> reference
+ * to the font map, so if all references to font map are dropped, the font
+ * map will be finalized even if there are fonts created with the font
+ * map that are still alive.  In that case this function will return %NULL.
+ * It is the responsibility of the user to ensure that the font map is kept
+ * alive.  In most uses this is not an issue as a #PangoContext holds
+ * a reference to the font map.
+ *
  * Return value: the #PangoFontMap for the font, or %NULL if @font is %NULL.
  *
  * Since: 1.10
@@ -1481,7 +1713,7 @@ pango_font_metrics_ref (PangoFontMetrics *metrics)
   if (metrics == NULL)
     return NULL;
 
-  metrics->ref_count++;
+  g_atomic_int_inc ((int *) &metrics->ref_count);
 
   return metrics;
 }
@@ -1502,9 +1734,7 @@ pango_font_metrics_unref (PangoFontMetrics *metrics)
 
   g_return_if_fail (metrics->ref_count > 0 );
 
-  metrics->ref_count--;
-
-  if (metrics->ref_count == 0)
+  if (g_atomic_int_dec_and_test ((int *) &metrics->ref_count))
     g_slice_free (PangoFontMetrics, metrics);
 }
 
@@ -1673,12 +1903,12 @@ pango_font_metrics_get_strikethrough_thickness (PangoFontMetrics *metrics)
 G_DEFINE_ABSTRACT_TYPE (PangoFontFamily, pango_font_family, G_TYPE_OBJECT)
 
 static void
-pango_font_family_class_init (PangoFontFamilyClass *class)
+pango_font_family_class_init (PangoFontFamilyClass *class G_GNUC_UNUSED)
 {
 }
 
 static void
-pango_font_family_init (PangoFontFamily *family)
+pango_font_family_init (PangoFontFamily *family G_GNUC_UNUSED)
 {
 }
 
@@ -1762,12 +1992,12 @@ pango_font_family_is_monospace (PangoFontFamily  *family)
 G_DEFINE_ABSTRACT_TYPE (PangoFontFace, pango_font_face, G_TYPE_OBJECT)
 
 static void
-pango_font_face_class_init (PangoFontFaceClass *class)
+pango_font_face_class_init (PangoFontFaceClass *class G_GNUC_UNUSED)
 {
 }
 
 static void
-pango_font_face_init (PangoFontFace *face)
+pango_font_face_init (PangoFontFace *face G_GNUC_UNUSED)
 {
 }
 
