@@ -577,6 +577,8 @@ shaper_font_cache_destroy (ShaperFontCache *cache)
 static void
 shaper_font_element_destroy (ShaperFontElement *element)
 {
+  if (element->shape_engine)
+    g_object_unref (element->shape_engine);
   if (element->font)
     g_object_unref (element->font);
   g_slice_free (ShaperFontElement, element);
@@ -588,7 +590,7 @@ get_shaper_font_cache (PangoFontset *fontset)
   ShaperFontCache *cache;
 
   static GQuark cache_quark = 0;
-  if (!cache_quark)
+  if (G_UNLIKELY (!cache_quark))
     cache_quark = g_quark_from_static_string ("pango-shaper-font-cache");
 
   cache = g_object_get_qdata (G_OBJECT (fontset), cache_quark);
@@ -632,8 +634,8 @@ shaper_font_cache_insert (ShaperFontCache   *cache,
 			  PangoFont         *font)
 {
   ShaperFontElement *element = g_slice_new (ShaperFontElement);
-  element->shape_engine = shape_engine;
-  element->font = font;
+  element->shape_engine = shape_engine ? g_object_ref (shape_engine) : NULL;
+  element->font = font ? g_object_ref (font) : NULL;
 
   g_hash_table_insert (cache->hash, GUINT_TO_POINTER (wc), element);
 }
@@ -1075,7 +1077,7 @@ get_shaper_and_font_foreach (PangoFontset *fontset G_GNUC_UNUSED,
       if (level != PANGO_COVERAGE_NONE)
 	{
 	  info->shape_engine = engine;
-	  info->font = g_object_ref (font);
+	  info->font = font;
 	  return TRUE;
 	}
     }
@@ -1131,28 +1133,34 @@ get_shaper_and_font (ItemizeState      *state,
   info.font = NULL;
 
   info.engines = state->exact_engines;
-  if (state->enable_fallback)
-    pango_fontset_foreach (state->current_fonts, get_shaper_and_font_foreach, &info);
-  else
-    get_shaper_and_font_foreach (NULL, get_base_font (state), &info);
-
-  if (info.shape_engine)
+  if (info.engines)
     {
-      *shape_engine = info.shape_engine;
-      *font = info.font;
-
-      /* skip caching if fallback disabled (see above) */
       if (state->enable_fallback)
-	shaper_font_cache_insert (state->cache, wc, *shape_engine, *font);
+	pango_fontset_foreach (state->current_fonts, get_shaper_and_font_foreach, &info);
+      else
+	get_shaper_and_font_foreach (NULL, get_base_font (state), &info);
 
-      return TRUE;
+      if (info.shape_engine)
+	{
+	  *shape_engine = info.shape_engine;
+	  *font = info.font;
+
+	  /* skip caching if fallback disabled (see above) */
+	  if (state->enable_fallback)
+	    shaper_font_cache_insert (state->cache, wc, *shape_engine, *font);
+
+	  return TRUE;
+	}
     }
 
   info.engines = state->fallback_engines;
-  if (state->enable_fallback)
-    pango_fontset_foreach (state->current_fonts, get_shaper_and_font_foreach, &info);
-  else
-    get_shaper_and_font_foreach (NULL, get_base_font (state), &info);
+  if (info.engines)
+    {
+      if (state->enable_fallback)
+	pango_fontset_foreach (state->current_fonts, get_shaper_and_font_foreach, &info);
+      else
+	get_shaper_and_font_foreach (NULL, get_base_font (state), &info);
+    }
 
   *shape_engine = info.shape_engine;
   *font = info.font;
@@ -1376,15 +1384,7 @@ itemize_state_process_run (ItemizeState *state)
 
 	  if (!g_object_get_data (G_OBJECT (fontmap), script_name))
 	    {
-	      const char *what;
-	      if (shape_engine == NULL)
-	        what = "shape engine";
-	      else if (font == NULL)
-	        what = "font";
-	      else
-	        what = "nothing (oops!)";
-	       
-	      g_warning ("failed to find shape engine, expect ugly output. engine-type='%s', script='%s'",
+	      g_warning ("failed to choose a font, expect ugly output. engine-type='%s', script='%s'",
 			 pango_font_map_get_shape_engine_type (fontmap),
 			 script_name);
 
