@@ -57,7 +57,7 @@ typedef struct _PangoFcFontPrivate PangoFcFontPrivate;
 struct _PangoFcFontPrivate
 {
   PangoFcDecoder *decoder;
-  gpointer context_key;
+  PangoFcFontKey *key;
   GUnicharToGlyphCacheEntry *char_to_glyph_cache;
 };
 
@@ -283,7 +283,7 @@ pango_fc_get_shaper_map (PangoLanguage *language)
 }
 
 static PangoEngineShape *
-pango_fc_font_find_shaper (PangoFont     *font,
+pango_fc_font_find_shaper (PangoFont     *font G_GNUC_UNUSED,
 			   PangoLanguage *language,
 			   guint32        ch)
 {
@@ -297,7 +297,7 @@ pango_fc_font_find_shaper (PangoFont     *font,
 
 static PangoCoverage *
 pango_fc_font_get_coverage (PangoFont     *font,
-			    PangoLanguage *language)
+			    PangoLanguage *language G_GNUC_UNUSED)
 {
   PangoFcFont *fcfont = (PangoFcFont *)font;
   PangoFcFontPrivate *priv = fcfont->priv;
@@ -754,6 +754,9 @@ _pango_fc_font_shutdown (PangoFcFont *font)
 
   if (PANGO_FC_FONT_GET_CLASS (font)->shutdown)
     PANGO_FC_FONT_GET_CLASS (font)->shutdown (font);
+
+  if (font->fontmap)
+    _pango_fc_font_map_remove (PANGO_FC_FONT_MAP (font->fontmap), font);
 }
 
 /**
@@ -855,21 +858,21 @@ _pango_fc_font_set_decoder (PangoFcFont    *font,
     g_object_ref (priv->decoder);
 }
 
-gpointer
-_pango_fc_font_get_context_key (PangoFcFont *fcfont)
+PangoFcFontKey *
+_pango_fc_font_get_font_key (PangoFcFont *fcfont)
 {
   PangoFcFontPrivate *priv = fcfont->priv;
 
-  return priv->context_key;
+  return priv->key;
 }
 
 void
-_pango_fc_font_set_context_key (PangoFcFont *fcfont,
-				gpointer     context_key)
+_pango_fc_font_set_font_key (PangoFcFont    *fcfont,
+			     PangoFcFontKey *key)
 {
   PangoFcFontPrivate *priv = fcfont->priv;
 
-  priv->context_key = context_key;
+  priv->key = key;
 }
 
 static FT_Glyph_Metrics *
