@@ -99,11 +99,10 @@ struct _PangoWin32FontMap
   GHashTable *families;
 
   /* Map LOGFONTWs (taking into account only the lfFaceName, lfItalic
-   * and lfWeight fields) to PangoWin32SizeInfo structs.
+   * and lfWeight fields) to LOGFONTWs corresponding to actual fonts
+   * installed.
    */
-  GHashTable *size_infos;
-
-  int n_fonts;
+  GHashTable *fonts;
 
   double resolution;		/* (points / pixel) * PANGO_SCALE */
 };
